@@ -403,7 +403,8 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
       atts += 2;
     }
     Me->InputInfo.m_iConversionID = 0;
-    
+    Me->InputInfo.m_strDefaultContext = ". ";
+
     return;
   }
 
@@ -649,6 +650,18 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     return;
   }
 
+  if(strcmp(name, "context") == 0) {
+    while(*atts != 0) {
+      if(strcmp(*atts, "default") == 0) {
+        atts++;
+        Me->InputInfo.m_strDefaultContext = *atts;
+        atts--;
+      }
+      atts += 2;
+    }
+    return;
+  }
+
   if(strcmp(name, "s") == 0) {
     AlphInfo::character NewCharacter;
 
