@@ -19,59 +19,43 @@ using namespace std;
 extern "C" gint key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 extern "C" gboolean button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data);
 extern "C" void realize_canvas(GtkWidget *widget, gpointer user_data);
-//extern "C" void speed_changed(GtkHScale *hscale, gpointer user_data);
 extern "C" gint canvas_configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer data);
 extern "C" gint key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data);
 extern "C" void canvas_destroy_event(GtkWidget *pWidget, gpointer pUserData);
-//extern "C" void alphabet_combo_changed(GtkWidget *pWidget, gpointer pUserData);
 extern "C" gboolean canvas_focus_event(GtkWidget *widget, GdkEventFocus *event, gpointer data);
+extern "C" gint canvas_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data);
+
+static bool g_iTimeoutID = 0;
 
 // CDasherControl class definitions
 CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl) {
+  m_pPangoCache = NULL;
+  m_pScreen = NULL;
+
   m_pDasherControl = pDasherControl;
   m_pVBox = GTK_WIDGET(pVBox);
 
   Realize();
+ 
+  m_pKeyboardHelper = new CKeyboardHelper(this);
+  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
+}
 
-  m_iComboCount = 0;
-    
-  // Start the dasher model
-
-  Start();        // FIXME - should we hold off on this until later?
-
-  // Create input device objects
-  // (We create the SocketInput object now even if socket input is not enabled, because
-  // we are not allowed to create it in response to a parameter update event later, because
-  // that would mean registering a new event listener during the processing of an event.
-
+void CDasherControl::CreateLocalFactories() {
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherMouseInput(m_pEventHandler, m_pSettingsStore)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CSocketInput(m_pEventHandler, m_pSettingsStore)));
-
-  m_pKeyboardHelper = new CKeyboardHelper;
-  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
-
-  CreateInput();
-
-//   m_pSocketInput = (CSocketInput *)GetModule(1);
-//   m_pSocketInput->Ref();
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasher1DMouseInput(m_pEventHandler, m_pSettingsStore)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInput(m_pEventHandler, m_pSettingsStore, this)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInputDiscrete(m_pEventHandler, m_pSettingsStore, this)));
+  
+  // Create locally cached copies of the mouse input objects, as we
+  // need to pass coordinates to them from the timer callback
   
   m_pMouseInput = (CDasherMouseInput *)GetModule(0);
   m_pMouseInput->Ref();
 
-  // Create a pango cache
-
-  m_pPangoCache = 0;
-
-  // TODO: Use system defaults?
-  if(GetStringParameter(SP_DASHER_FONT) == "")
-    SetStringParameter(SP_DASHER_FONT, "Sans 10");
-
- 
-  m_pPangoCache = new CPangoCache(GetStringParameter(SP_DASHER_FONT));
-
-  // Don't create the screen until we've been realised.
-
-  m_pScreen = NULL;
+  m_p1DMouseInput = (CDasher1DMouseInput *)GetModule(2);
+  m_p1DMouseInput->Ref();
 
 }
 
@@ -82,40 +66,11 @@ void CDasherControl::SetupUI() {
 
   GtkWidget *pFrame = gtk_frame_new(NULL);
   gtk_frame_set_shadow_type(GTK_FRAME(pFrame), GTK_SHADOW_IN); 
-  
-//   m_pSpeedFrame = gtk_frame_new("Speed:");
-
-//   m_pSpeedHScale = gtk_hscale_new_with_range(0.1, 8.0, 0.1);
-//   gtk_scale_set_digits( GTK_SCALE(m_pSpeedHScale), 1 );
-
-//   gtk_container_add(GTK_CONTAINER(m_pSpeedFrame), m_pSpeedHScale);
   gtk_container_add(GTK_CONTAINER(pFrame), m_pCanvas);
 
-
-//   m_pStatusBar = gtk_hbox_new(false, 2);
-
-//   m_pSpin = gtk_spin_button_new_with_range(0.1, 8.0, 0.1);
-//   m_pCombo = gtk_combo_box_new_text();
-
-//   gtk_widget_set_size_request(m_pCombo, 256, -1);
-
-//   m_pStatusLabel = gtk_label_new("Characters/min: --");
-//   gtk_label_set_justify(GTK_LABEL(m_pStatusLabel), GTK_JUSTIFY_RIGHT);
-
-//   gtk_box_pack_start(GTK_BOX(m_pStatusBar), gtk_label_new("Speed:"), 0, 0, 0);
-//   //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pSpin, 0, 0, 0);
-//   //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pCombo, 0, 0, 0);
-//   //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pStatusLabel, TRUE, TRUE, 0);
-
-   gtk_box_pack_start(GTK_BOX(m_pVBox), pFrame, TRUE, TRUE, 0);
-//   //  gtk_box_pack_start(GTK_BOX(m_pVBox), m_pSpeedFrame, FALSE, FALSE, 0);
-//   gtk_box_pack_start(GTK_BOX(m_pVBox), m_pStatusBar, FALSE, FALSE, 0);
-
+  gtk_box_pack_start(GTK_BOX(m_pVBox), pFrame, TRUE, TRUE, 0);
   gtk_widget_show_all(GTK_WIDGET(m_pVBox));
 
-//   if(!GetBoolParameter(BP_SHOW_SLIDER))
-//     gtk_widget_hide(m_pStatusBar);
-
   // Connect callbacks - note that we need to implement the callbacks
   // as "C" style functions and pass this as user data so they can
   // call the object
@@ -126,15 +81,23 @@ void CDasherControl::SetupUI() {
   g_signal_connect(m_pCanvas, "configure_event", G_CALLBACK(canvas_configure_event), this);
   g_signal_connect(m_pCanvas, "destroy", G_CALLBACK(canvas_destroy_event), this);
 
-  // We'll use the same call back for keyboard events from the canvas
-  // and slider - maybe this isn't the right thing to do long term
-
   g_signal_connect(m_pCanvas, "key-release-event", G_CALLBACK(key_release_event), this);
   g_signal_connect(m_pCanvas, "key_press_event", G_CALLBACK(key_press_event), this);
 
   g_signal_connect(m_pCanvas, "focus_in_event", G_CALLBACK(canvas_focus_event), this);
+  g_signal_connect(m_pCanvas, "expose_event", G_CALLBACK(canvas_expose_event), this);
+
+  // Create the Pango cache
+
+  // TODO: Use system defaults?
+  if(GetStringParameter(SP_DASHER_FONT) == "")
+    SetStringParameter(SP_DASHER_FONT, "Sans 10");
+ 
+  m_pPangoCache = new CPangoCache(GetStringParameter(SP_DASHER_FONT));
+
 }
 
+
 void CDasherControl::SetupPaths() {
   char *home_dir;
   char *user_data_dir;
@@ -167,9 +130,8 @@ void CDasherControl::ScanAlphabetFiles(std::vector<std::string> &vFileList) {
 
   if(directory) {
     while((filename = g_dir_read_name(directory))) {
-      if(alphabet_filter(filename, alphabetglob)) {
+      if(g_pattern_match_string(alphabetglob, filename)) 
 	vFileList.push_back(filename);
-      }
     }
     g_dir_close(directory);
   }
@@ -178,13 +140,13 @@ void CDasherControl::ScanAlphabetFiles(std::vector<std::string> &vFileList) {
 
   if(directory) {
     while((filename = g_dir_read_name(directory))) {
-      if(alphabet_filter(filename, alphabetglob)) {
+      if(g_pattern_match_string(alphabetglob, filename))
 	vFileList.push_back(filename);
-      }
     }
     g_dir_close(directory);
   }
-  // FIXME - need to delete glob?
+
+  g_pattern_spec_free(alphabetglob);
 }
 
 void CDasherControl::ScanColourFiles(std::vector<std::string> &vFileList) {
@@ -198,9 +160,8 @@ void CDasherControl::ScanColourFiles(std::vector<std::string> &vFileList) {
 
   if(directory) {
     while((filename = g_dir_read_name(directory))) {
-      if(colour_filter(filename, colourglob)) {
+      if(g_pattern_match_string(colourglob, filename))
 	vFileList.push_back(filename);
-      }
     }
     g_dir_close(directory);
   }
@@ -209,33 +170,27 @@ void CDasherControl::ScanColourFiles(std::vector<std::string> &vFileList) {
 
   if(directory) {
     while((filename = g_dir_read_name(directory))) {
-      if(colour_filter(filename, colourglob)) {
+      if(g_pattern_match_string(colourglob, filename))
 	vFileList.push_back(filename);
-      }
     }
     g_dir_close(directory);
   }
 
-  // FIXME - need to delete glob?
+  g_pattern_spec_free(colourglob);
 }
 
 CDasherControl::~CDasherControl() {
-
-  WriteTrainFileFull();
-
-  // Delete the input devices
-
-  if(m_pMouseInput != NULL) {
+  if(m_pMouseInput) {
     m_pMouseInput->Unref();
     m_pMouseInput = NULL;
   }
 
-//   if(m_pSocketInput != NULL) {
-//     m_pSocketInput->Unref();
-//     m_pSocketInput = NULL;
-//   }
+  if(m_p1DMouseInput) {
+    m_p1DMouseInput->Unref();
+    m_p1DMouseInput = NULL;
+  }
 
-  if(m_pPangoCache != NULL) {
+  if(m_pPangoCache) {
     delete m_pPangoCache;
     m_pPangoCache = NULL;
   }
@@ -266,9 +221,7 @@ void CDasherControl::SetFocus() {
 }
 
 GArray *CDasherControl::GetAllowedValues(int iParameter) {
-
-  // FIXME - this should really be implemented in DasherInterface in
-  // place of GetAlphabets and GetColours
+  // Glib version of the STL based core function
 
   GArray *pRetVal(g_array_new(false, false, sizeof(gchar *)));
 
@@ -287,11 +240,27 @@ GArray *CDasherControl::GetAllowedValues(int iParameter) {
   return pRetVal;
 }
 
-void CDasherControl::RealizeCanvas() {
+void CDasherControl::RealizeCanvas(GtkWidget *pWidget) {
+  // TODO: Pointless function - call directly from C callback.
+  OnUIRealised();
+}
+
+void CDasherControl::StartTimer() {
   // Start the timer loops as everything is set up
   // Aim for 20 frames per second
 
-  g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 50, timer_callback, this, NULL);
+  if(g_iTimeoutID == 0) {
+    g_iTimeoutID = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 50, timer_callback, this, NULL);
+    // TODO: Reimplement this (or at least reimplement some kind of status reporting)
+    //g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 5000, long_timer_callback, this, NULL);
+  }
+}
+
+void CDasherControl::ShutdownTimer() {
+  // TODO: Figure out how to implement this - at the moment it's done
+  // through a return value from the timer callback, but it would be
+  // nicer to prevent any further calls as soon as the shutdown signal
+  // has been receieved.
 }
 
 int CDasherControl::CanvasConfigureEvent() {
@@ -306,17 +275,68 @@ int CDasherControl::CanvasConfigureEvent() {
 }
 
 void CDasherControl::ExternalEventHandler(Dasher::CEvent *pEvent) {
-  // Pass events outside
-  if(pEvent->m_iEventType == 1) {
+  // Convert events coming from the core to Glib signals.
+
+  if(pEvent->m_iEventType == EV_PARAM_NOTIFY) {
     Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
     HandleParameterNotification(pEvt->m_iParameter);
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_changed", pEvt->m_iParameter);
   }
-  // TODO: Horrible - just keep events here
-  else if((pEvent->m_iEventType >= 2) && (pEvent->m_iEventType <= 8)) {
-    HandleEvent(pEvent);
+  else if(pEvent->m_iEventType == EV_EDIT) {
+    CEditEvent *pEditEvent(static_cast < CEditEvent * >(pEvent));
+    
+    if(pEditEvent->m_iEditType == 1) {
+      // Insert event
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_insert", pEditEvent->m_sText.c_str());
+    }
+    else if(pEditEvent->m_iEditType == 2) {
+      // Delete event
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_delete", pEditEvent->m_sText.c_str());
+    }
+    else if(pEditEvent->m_iEditType == 10) {
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_convert");
+    }
+    else if(pEditEvent->m_iEditType == 11) {
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_protect");
+    }
   }
+  else if(pEvent->m_iEventType == EV_EDIT_CONTEXT) {
+    CEditContextEvent *pEditContextEvent(static_cast < CEditContextEvent * >(pEvent));
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_context_request", pEditContextEvent->m_iMaxLength);
+  }
+  else if(pEvent->m_iEventType == EV_START) {
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_start");
+  }
+  else if(pEvent->m_iEventType == EV_STOP) {
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_stop");
+  }
+  else if(pEvent->m_iEventType == EV_CONTROL) {
+    CControlEvent *pControlEvent(static_cast < CControlEvent * >(pEvent));
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_control", pControlEvent->m_iID);
+  }
+  else if(pEvent->m_iEventType == EV_LOCK) {
+    CLockEvent *pLockEvent(static_cast<CLockEvent *>(pEvent));
+    DasherLockInfo sInfo;
+    sInfo.szMessage = pLockEvent->m_strMessage.c_str();
+    sInfo.bLock = pLockEvent->m_bLock;
+    sInfo.iPercent = pLockEvent->m_iPercent;
 
-}
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_lock_info", &sInfo);
+  }
+  else if(pEvent->m_iEventType == EV_MESSAGE) {
+    CMessageEvent *pMessageEvent(static_cast<CMessageEvent *>(pEvent));
+    DasherMessageInfo sInfo;
+    sInfo.szMessage = pMessageEvent->m_strMessage.c_str();
+    sInfo.iID = pMessageEvent->m_iID;
+    sInfo.iType = pMessageEvent->m_iType;
+
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_message", &sInfo);
+  }
+  else if(pEvent->m_iEventType == EV_COMMAND) {
+    CCommandEvent *pCommandEvent(static_cast<CCommandEvent *>(pEvent));
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_command", pCommandEvent->m_strCommand.c_str());
+  }
+};
 
 void CDasherControl::WriteTrainFile(const std::string &strNewText) {
   if(strNewText.length() == 0)
@@ -348,12 +368,11 @@ void CDasherControl::ExternalKeyUp(int iKeyVal) {
 }
 
 void CDasherControl::HandleParameterNotification(int iParameter) {
-
   switch(iParameter) {
   case SP_DASHER_FONT:
     if(m_pPangoCache) {
       m_pPangoCache->ChangeFont(GetStringParameter(SP_DASHER_FONT));
-      Redraw(true);
+      ScheduleRedraw();
     }
     break;
   case BP_GLOBAL_KEYBOARD:
@@ -361,75 +380,32 @@ void CDasherControl::HandleParameterNotification(int iParameter) {
       m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
     break;
   }
-
-  // Emit a dasher_changed signal to notify the application about changes.
-  g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_changed", iParameter);
 }
 
-
-void CDasherControl::HandleEvent(CEvent *pEvent) {
-  if(pEvent->m_iEventType == 2) {
-    CEditEvent *pEditEvent(static_cast < CEditEvent * >(pEvent));
-
-    if(pEditEvent->m_iEditType == 1) {
-      // Insert event
-      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_insert", pEditEvent->m_sText.c_str());
-    }
-    else if(pEditEvent->m_iEditType == 2) {
-      // Delete event
-      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_delete", pEditEvent->m_sText.c_str());
-    }
-    else if(pEditEvent->m_iEditType == 10) {
-      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_convert");
-    }
-    else if(pEditEvent->m_iEditType == 11) {
-      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_protect");
-    }
-  }
-  else if(pEvent->m_iEventType == 3) {
-    CEditContextEvent *pEditContextEvent(static_cast < CEditContextEvent * >(pEvent));
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_context_request", pEditContextEvent->m_iMaxLength);
-  }
-  else if(pEvent->m_iEventType == 4) {
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_start");
-  }
-  else if(pEvent->m_iEventType == 5) {
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_stop");
-  }
-  else if(pEvent->m_iEventType == 6) {
-    CControlEvent *pControlEvent(static_cast < CControlEvent * >(pEvent));
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_control", pControlEvent->m_iID);
-  }
-  else if(pEvent->m_iEventType == 7) {
-    CLockEvent *pLockEvent(static_cast<CLockEvent *>(pEvent));
-    DasherLockInfo sInfo;
-    sInfo.szMessage = pLockEvent->m_strMessage.c_str();
-    sInfo.bLock = pLockEvent->m_bLock;
-    sInfo.iPercent = pLockEvent->m_iPercent;
-
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_lock_info", &sInfo);
-  }
-  else if(pEvent->m_iEventType == 8) {
-    CMessageEvent *pMessageEvent(static_cast<CMessageEvent *>(pEvent));
-    DasherMessageInfo sInfo;
-    sInfo.szMessage = pMessageEvent->m_strMessage.c_str();
-    sInfo.iID = pMessageEvent->m_iID;
-    sInfo.iType = pMessageEvent->m_iType;
-
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_message", &sInfo);
-  }
-};
-
 int CDasherControl::TimerEvent() {
   int x, y;
 
   gdk_window_get_pointer(m_pCanvas->window, &x, &y, NULL);
   m_pMouseInput->SetCoordinates(x, y);
 
-  NewFrame(get_time());
+  gdk_window_get_pointer(gdk_get_default_root_window(), &x, &y, NULL);
+
+  int iRootWidth;
+  int iRootHeight;
+
+  gdk_drawable_get_size(gdk_get_default_root_window(), &iRootWidth, &iRootHeight);
+
+  if(GetLongParameter(LP_YSCALE) < 10)
+    SetLongParameter(LP_YSCALE, 10);
+
+  y = (y - iRootHeight / 2) * 4096 / GetLongParameter(LP_YSCALE);
+
+  m_p1DMouseInput->SetCoordinates(x, y);
+
+  NewFrame(get_time(), false);
 
   // Update our UserLog object about the current mouse position
-  CUserLog* pUserLog = GetUserLogPtr();
+  CUserLogBase* pUserLog = GetUserLogPtr();
   if (pUserLog != NULL) {  
       // We want current canvas and window coordinates so normalization
       // is done properly with respect to the canvas.
@@ -463,6 +439,17 @@ int CDasherControl::TimerEvent() {
   // See CVS for code which used to be here
 }
 
+int CDasherControl::LongTimerEvent() {
+//   std::cout << "Framerate: " << GetFramerate() << std::endl;
+//   std::cout << "Render count: " << GetRenderCount() << std::endl;
+  return 1;
+}
+
+gboolean CDasherControl::ExposeEvent() {
+  NewFrame(get_time(), true);
+  return 0;
+}
+
 gboolean CDasherControl::ButtonPressEvent(GdkEventButton *event) {
 
   // Take the focus if we click on the canvas
@@ -477,33 +464,33 @@ gboolean CDasherControl::ButtonPressEvent(GdkEventButton *event) {
 
   gtk_widget_grab_focus(GTK_WIDGET(m_pCanvas));
   g_signal_emit_by_name(GTK_OBJECT(m_pCanvas), "focus_in_event", GTK_WIDGET(m_pCanvas), focusEvent, NULL, &returnType);
-
+  
   if(event->type == GDK_BUTTON_PRESS)
-    KeyDown(get_time(), 100);
+    HandleClickDown(get_time(), (int)event->x, (int)event->y);
   else if(event->type == GDK_BUTTON_RELEASE)
-    KeyUp(get_time(), 100);
+    HandleClickUp(get_time(), (int)event->x, (int)event->y);
 
   return false;
 }
 
 gint CDasherControl::KeyReleaseEvent(GdkEventKey *event) {
-
+  // TODO: This is seriously flawed - the semantics of of X11 Keyboard
+  // events mean the there's no guarantee that key up/down events will
+  // be received in pairs.
 
   if((event->keyval == GDK_Shift_L) || (event->keyval == GDK_Shift_R)) {
-    if(event->state & GDK_CONTROL_MASK)
-      SetLongParameter(LP_BOOSTFACTOR, 25);
-    else
-      SetLongParameter(LP_BOOSTFACTOR, 100);
+//     if(event->state & GDK_CONTROL_MASK)
+//       SetLongParameter(LP_BOOSTFACTOR, 25);
+//     else
+//       SetLongParameter(LP_BOOSTFACTOR, 100);
   }
   else if((event->keyval == GDK_Control_L) || (event->keyval == GDK_Control_R)) {
-    if(event->state & GDK_SHIFT_MASK)
-      SetLongParameter(LP_BOOSTFACTOR, 175);
-    else
-      SetLongParameter(LP_BOOSTFACTOR, 100);
+//     if(event->state & GDK_SHIFT_MASK)
+//       SetLongParameter(LP_BOOSTFACTOR, 175);
+//     else
+//       SetLongParameter(LP_BOOSTFACTOR, 100);
   }
   else {
-    int iKeyVal;
-
     if(m_pKeyboardHelper) {
       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
@@ -516,20 +503,18 @@ gint CDasherControl::KeyReleaseEvent(GdkEventKey *event) {
 }
 
 gint CDasherControl::KeyPressEvent(GdkEventKey *event) {
-  if((event->keyval == GDK_Shift_L) || (event->keyval == GDK_Shift_R))
-    SetLongParameter(LP_BOOSTFACTOR, 175);
-  else if((event->keyval == GDK_Control_L) || (event->keyval == GDK_Control_R))
-    SetLongParameter(LP_BOOSTFACTOR, 25);
-  else {
-    int iKeyVal;
-
+  //  if((event->keyval == GDK_Shift_L) || (event->keyval == GDK_Shift_R))
+    //    SetLongParameter(LP_BOOSTFACTOR, 175);
+  //  else if((event->keyval == GDK_Control_L) || (event->keyval == GDK_Control_R))
+  //   SetLongParameter(LP_BOOSTFACTOR, 25);
+  // else {
     if(m_pKeyboardHelper) {
       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
       if(iKeyVal != -1)
 	KeyDown(get_time(), iKeyVal);
     }
-  }
+    //  }
   return 0;
 }
 
@@ -545,7 +530,7 @@ void CDasherControl::CanvasDestroyEvent() {
 // Tell the logging object that a new user trial is starting.
 void CDasherControl::UserLogNewTrial()
 {
-  CUserLog* pUserLog = GetUserLogPtr();
+  CUserLogBase* pUserLog = GetUserLogPtr();
   if (pUserLog != NULL) { 
     pUserLog->NewTrial();
   }
@@ -560,16 +545,6 @@ int CDasherControl::GetFileSize(const std::string &strFileName) {
     return 0;
 }
 
-// FIXME - these two methods seem a bit pointless!
-
-int CDasherControl::alphabet_filter(const gchar *filename, GPatternSpec *alphabetglob) {
-  return int (g_pattern_match_string(alphabetglob, filename));
-}
-
-int CDasherControl::colour_filter(const gchar *filename, GPatternSpec *colourglob) {
-  return int (g_pattern_match_string(colourglob, filename));
-}
-
 // "C" style callbacks - these are here just because it's not possible
 // (or at least not easy) to connect a callback directly to a C++
 // method, so we pass a pointer to th object in the user_data field
@@ -577,7 +552,7 @@ int CDasherControl::colour_filter(const gchar *filename, GPatternSpec *colourglo
 // here.
 
 extern "C" void realize_canvas(GtkWidget *widget, gpointer user_data) {
-  static_cast < CDasherControl * >(user_data)->RealizeCanvas();
+  static_cast < CDasherControl * >(user_data)->RealizeCanvas(widget);
 }
 
 
@@ -604,3 +579,7 @@ extern "C" gint key_release_event(GtkWidget *pWidget, GdkEventKey *event, gpoint
 extern "C" gboolean canvas_focus_event(GtkWidget *widget, GdkEventFocus *event, gpointer data) {
   return static_cast < CDasherControl * >(data)->FocusEvent(widget, event);
 }
+
+extern "C" gint canvas_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
+  return ((CDasherControl*)data)->ExposeEvent();
+}
