@@ -23,20 +23,6 @@ extern GtkWidget *append_filesel;
 extern GtkWidget *window;
 extern GtkWidget *file_selector;
 
-void PopulateMenus(GladeXML *pGladeWidgets) {
-  switch(gtk_dasher_control_get_parameter_long( GTK_DASHER_CONTROL(pDasherWidget), LP_DASHER_FONTSIZE)) {
-  case 1:
-    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(glade_xml_get_widget(pGladeWidgets, "fontsizenormal")), true);
-    break;
-  case 2:
-    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(glade_xml_get_widget(pGladeWidgets, "fontsizelarge")), true);
-    break;
-  case 4:
-    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(glade_xml_get_widget(pGladeWidgets, "fontsizevlarge")), true);
-    break;
-  }
-}
-
 extern "C" void select_save_file_as(GtkWidget * widget, gpointer user_data);
 
 // 'File' Menu
@@ -333,9 +319,10 @@ extern "C" bool ask_save_before_exit(GtkWidget *widget, gpointer data) {
     // FIXME - REIMPLEMENT
 
     //    write_to_file();
+
     gtk_main_quit();
   }
-  return false;
+  return true;
 }
 
 // 'Edit' menu
@@ -356,6 +343,10 @@ extern "C" void clipboard_copy_all(void) {
   dasher_editor_clipboard(g_pEditor, CLIPBOARD_COPYALL);
 }
 
+extern "C" void on_menututorial_activate() {
+  dasher_editor_start_tutorial(g_pEditor);
+}
+
 // Not actually on the menu, but should be?
 
 extern "C" void clipboard_select_all(void) {
@@ -369,7 +360,7 @@ extern "C" void clipboard_select_all(void) {
 // Font selector options will be in their own file
 
 extern "C" void set_dasher_fontsize(GtkWidget *widget, gpointer user_data) {
-  if(GTK_CHECK_MENU_ITEM(widget)->active == TRUE) {
+  if(GTK_TOGGLE_BUTTON(widget)->active == TRUE) {
     if(!strcmp(gtk_widget_get_name(GTK_WIDGET(widget)), "fontsizenormal")) {
       gtk_dasher_control_set_parameter_long(GTK_DASHER_CONTROL(pDasherWidget), LP_DASHER_FONTSIZE, Dasher::Opts::Normal);
     }
@@ -391,7 +382,7 @@ extern "C" void reset_fonts(GtkWidget *widget, gpointer user_data) {
 
 extern "C" void about_dasher(GtkWidget *widget, gpointer user_data) {
 
-#ifdef GNOME_LIBS
+#if (defined GNOME_LIBS) || (GTK_CHECK_VERSION(2,6,0))
 
   // In alphabetical order
   const gchar *authors[] = {
@@ -420,6 +411,7 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data) {
     NULL
   };
 
+#if GTK_CHECK_VERSION(2,6,0)
   gtk_show_about_dialog(GTK_WINDOW(window),
 			"authors", authors,
 			"comments", _("Dasher is a predictive text entry application"), 
@@ -432,6 +424,23 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data) {
 			"website", "http://www.dasher.org.uk/",
 			"wrap-license", true,
 			NULL);
+#else
+  gchar *translator_credits = _("translator-credits");
+
+  GtkWidget *about = gnome_about_new (_("Dasher"), 
+			   PACKAGE_VERSION, 
+			   "Copyright The Dasher Project\n",
+			   _("Dasher is a predictive text entry application"),
+			   (const char **)authors,
+			   (const char **)documenters,
+			   strcmp (translator_credits, "translator-credits") != 0 ? (const char *)translator_credits : NULL,
+			   NULL);
+  
+  gtk_window_set_transient_for (GTK_WINDOW(about), GTK_WINDOW (window));
+  //  g_signal_connect (G_OBJECT (about), "destory", G_CALLBACK (gtk_widget_destroyed), &about);
+  gtk_widget_show(about);
+
+#endif
   
 #else
   // EAT UGLY ABOUT BOX, PHILISTINE
@@ -448,7 +457,7 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data) {
   gtk_widget_show(label);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, FALSE, FALSE, 0);
 
-  label = gtk_label_new("http://www.inference.phy.cam.ac.uk/dasher/");
+  label = gtk_label_new("http://www.dasher.org.uk/");
   gtk_widget_show(label);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, FALSE, FALSE, 0);
 
