@@ -18,7 +18,7 @@
 #include "../../DasherCore/Event.h"
 #include "FilenameGUI.h"
 #include "../resource.h"
-#include "../DasherInterface.h"
+#include "../../DasherCore/DasherInterfaceBase.h"
 #include "../ActionSpeech.h"
 
 #include "../Common/DasherEncodingToCP.h"
@@ -1066,4 +1066,4 @@ void CEdit::HandleStop() {
 
   if(m_pAppSettings->GetBoolParameter(APP_BP_COPY_ALL_ON_STOP))
     CopyAll();
-}
+}
\ No newline at end of file
