@@ -11,7 +11,7 @@
 #include "Screen.h"
 
 #include "../Common/DasherEncodingToCP.h"
-#include "../../DasherCore/CustomColours.h"
+// #include "../../DasherCore/CustomColours.h"
 using namespace WinLocalisation;
 using namespace Dasher;
 using namespace Opts;
@@ -87,7 +87,7 @@ CScreen::~CScreen() {
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CScreen::SetInterface(CDasherInterface *DasherInterface) {
+void CScreen::SetInterface(Dasher::CDasherInterfaceBase *DasherInterface) {
   DASHER_ASSERT_VALIDPTR_RW(DasherInterface);
 
  // CDasherScreen::SetInterface(DasherInterface);
@@ -95,7 +95,7 @@ void CScreen::SetInterface(CDasherInterface *DasherInterface) {
 //  CodePage = EncodingToCP(m_pDasherInterface->GetAlphabetType());
 }
 
-void CScreen::SetColourScheme(const Dasher::CCustomColours *pColours) {
+void CScreen::SetColourScheme(const Dasher::CColourIO::ColourInfo *pColours) {
   m_cPens.clear();
   m_cBrushes.clear();
   m_pColours = pColours;
@@ -158,7 +158,8 @@ void CScreen::DrawString(const std::string &OutputString, Dasher::screenint x1,
   COLORREF iCRefOld;
   COLORREF iCRefNew;
 
-  iCRefNew = RGB(m_pColours->GetRed(4), m_pColours->GetGreen(4), m_pColours->GetBlue(4));
+  // TODO: Hardcoded numbers
+  iCRefNew = RGB(m_pColours->Reds[4], m_pColours->Greens[4], m_pColours->Blues[4]);
   
   iCRefOld = SetTextColor(m_hDCBuffer, iCRefNew);
 
